# Generated by Django 4.2.11 on 2024-04-29 07:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('capeditor', '0004_predefinedalertarea'),
    ]

    operations = [
        migrations.AlterField(
            model_name='capsetting',
            name='sender',
            field=models.CharField(blank=True, help_text='Email of the sending institution', max_length=255, null=True, verbose_name='CAP Sender Email'),
        ),
    ]
