# Generated by Django 4.1.7 on 2023-07-12 09:55

from django.db import migrations, models
import django.db.models.deletion
import wagtail.blocks
import wagtail.fields
import wagtailiconchooser.blocks


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0083_workflowcontenttype'),
    ]

    operations = [
        migrations.CreateModel(
            name='CapSetting',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sender', models.CharField(blank=True, help_text='Can be the website link or email of the sending institution', max_length=255, null=True, verbose_name='CAP Sender Identifier')),
                ('sender_name', models.CharField(blank=True, help_text='Name of the sending institution', max_length=255, null=True, verbose_name='CAP Sender Name')),
                ('contacts', wagtail.fields.StreamField([('contact', wagtail.blocks.StructBlock([('contact', wagtail.blocks.CharBlock(label='Contact Detail', max_length=255))], label='Contact'))], blank=True, help_text='Contact for follow-up and confirmation of the alert message', null=True, use_json_field=True, verbose_name='Contact Details')),
                ('hazard_types', wagtail.fields.StreamField([('hazard_type', wagtail.blocks.StructBlock([('hazard', wagtail.blocks.CharBlock(help_text='Name of Hazard', label='Hazard', max_length=255)), ('icon', wagtailiconchooser.blocks.IconChooserBlock(required=False))], label='Hazard Type'))], blank=True, help_text='Hazards monitored by the institution', null=True, use_json_field=True, verbose_name='Hazard Types')),
                ('audience_types', wagtail.fields.StreamField([('audience_type', wagtail.blocks.StructBlock([('audience', wagtail.blocks.CharBlock(help_text='Intended audience', label='Audience', max_length=255))], label='Audience Type'))], blank=True, help_text='Target audiences for published alerts', null=True, use_json_field=True, verbose_name='Audience Types')),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.site')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
